#!/usr/bin/env python
"""

Create a plotsgy input file.  Hopefully over time, this program will
get smarter and be able to guess from a segy file what the paramters
should be.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: makepltsegy.py,v 1.6 2005/09/05 16:42:05 schwehr Exp $

TODO:

* From a segy file, inspect the shot numbers and the range of shot times to set tbi,
  tfi, and timscal to some appropriate starting point.

http://schwehr.org/software/segy-py/
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.6 $' # See man ident
__date__ = '$Date: 2005/09/05 16:42:05 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """Alistair Harding for making pltsegy"""

# Will hopefully need the segy module in the future
#from segy import Segy
#import segy
import os
import sys

from optparse import OptionParser
vcvs = "$Revision: 1.6 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-o','--out',dest='pltsegyFile',default='default.pltsegy', #None, FIX: Force the user to give one
		    help='file to write the pltsegy file to.  If none, then the name will be based on the input file.')

myparser.add_option('-p','--postscript',dest='postscriptFile',default='default.ps',
		    help='plfile for postscript output  If none, then the name will be based on the input file.')

myparser.add_option('-i','--input-segy',dest='segyFile',default='line.sgy',#None,
		    help='dtfile.  Input SEG-Y file.')
# Do not give the option for the segy file.
myparser.add_option('-X','--x11-display',dest='x11',default=True,action='store_true',
                    help='Display output to X11 window. device=16  [ default ]')
myparser.add_option('-P','--ps-display',dest='x11',action='store_false',
                    help='Output to postscript file.    device=15')

myparser.add_option('-s','--shb',type='int',dest='shb',default=0,     help='Beginning shot to plot [default: %default]')
myparser.add_option('-S','--shf',type='int',dest='shf',default=999999,help='Finishing shot to plot [default: %default]')

myparser.add_option('-t','--tbi',type='float',dest='tbi',default=0.00,help='Beginning time to plot (sec) [default: %default]')
myparser.add_option('-T','--tfi',type='float',dest='tfi',default=0.75,help='Finishing time to plot (sec) [default: %default]')

myparser.add_option('--timscal','--timescale',type='int',dest='timscal',default=1000,help='see man page [default: %default]')
myparser.add_option('--tmark',type='float',dest='tmark',default=0.01,help='see man page [default: %default]')

myparser.add_option('--spa',type='float',dest='spa',default=0.025,help='horizontal shot spacing [default: %default]')

myparser.add_option('--title',dest='title',default=None,help='put a text label on the plot [default: %default]')

myparser.add_option('--anninc',dest='anninc',type='int',default=500,
                    help='how many shots to increment before labeling shots [default: %default]')



def writeLevel(o):
    o.write("""       level =       
        0.007874
        0.015748
        0.023622
        0.031496
        0.03937
        0.047244
        0.055118
        0.062992
        0.070866
        0.07874
        0.086614
        0.094488
        0.10236
        0.11024
        0.11811
        0.12598
        0.13386
        0.14173
        0.14961
        0.15748
        0.16535
        0.17323
        0.1811
        0.18898
        0.19685
        0.20472
        0.2126
        0.22047
        0.22835
        0.23622
        0.24409
        0.25197
        0.25984
        0.26772
        0.27559
        0.28346
        0.29134
        0.29921
        0.30709
        0.31496
        0.32283
        0.33071
        0.33858
        0.34646
        0.35433
        0.3622
        0.37008
        0.37795
        0.38583
        0.3937
        0.40157
        0.40945
        0.41732
        0.4252
        0.43307
        0.44094
        0.44882
        0.45669
        0.46457
        0.47244
        0.48031
        0.48819
        0.49606
        0.50394
        0.51181
        0.51969
        0.52756
        0.53543
        0.54331
        0.55118
        0.55906
        0.56693
        0.5748
        0.58268
        0.59055
        0.59843
        0.6063
        0.61417
        0.62205
        0.62992
        0.6378
        0.64567
        0.65354
        0.66142
        0.66929
        0.67717
        0.68504
        0.69291
        0.70079
        0.70866
        0.71654
        0.72441
        0.73228
        0.74016
        0.74803
        0.75591
        0.76378
        0.77165
        0.77953
        0.7874
        0.79528
        0.80315
        0.81102
        0.8189
        0.82677
        0.83465
        0.84252
        0.85039
        0.85827
        0.86614
        0.87402
        0.88189
        0.88976
        0.89764
        0.90551
        0.91339
        0.92126
        0.92913
        0.93701
        0.94488
        0.95276
        0.96063
        0.9685
        0.97638
        0.98425
        0.99213
        1;
""")
    return

def writePalette(o):
    o.write("""       palette =     
  255.0000  255.0000  255.0000
  253.0078  253.0078  253.0078
  251.0156  251.0156  251.0156
  249.0234  249.0234  249.0234
  247.0312  247.0312  247.0312
  245.0391  245.0391  245.0391
  243.0469  243.0469  243.0469
  241.0547  241.0547  241.0547
  239.0625  239.0625  239.0625
  237.0703  237.0703  237.0703
  235.0781  235.0781  235.0781
  233.0859  233.0859  233.0859
  231.0938  231.0938  231.0938
  229.1016  229.1016  229.1016
  227.1094  227.1094  227.1094
  225.1172  225.1172  225.1172
  223.1250  223.1250  223.1250
  221.1328  221.1328  221.1328
  219.1406  219.1406  219.1406
  217.1484  217.1484  217.1484
  215.1562  215.1562  215.1562
  213.1641  213.1641  213.1641
  211.1719  211.1719  211.1719
  209.1797  209.1797  209.1797
  207.1875  207.1875  207.1875
  205.1953  205.1953  205.1953
  203.2031  203.2031  203.2031
  201.2109  201.2109  201.2109
  199.2188  199.2188  199.2188
  197.2266  197.2266  197.2266
  195.2344  195.2344  195.2344
  193.2422  193.2422  193.2422
  191.2500  191.2500  191.2500
  189.2578  189.2578  189.2578
  187.2656  187.2656  187.2656
  185.2734  185.2734  185.2734
  183.2812  183.2812  183.2812
  181.2891  181.2891  181.2891
  179.2969  179.2969  179.2969
  177.3047  177.3047  177.3047
  175.3125  175.3125  175.3125
  173.3203  173.3203  173.3203
  171.3281  171.3281  171.3281
  169.3359  169.3359  169.3359
  167.3438  167.3438  167.3438
  165.3516  165.3516  165.3516
  163.3594  163.3594  163.3594
  161.3672  161.3672  161.3672
  159.3750  159.3750  159.3750
  157.3828  157.3828  157.3828
  155.3906  155.3906  155.3906
  153.3984  153.3984  153.3984
  151.4062  151.4062  151.4062
  149.4141  149.4141  149.4141
  147.4219  147.4219  147.4219
  145.4297  145.4297  145.4297
  143.4375  143.4375  143.4375
  141.4453  141.4453  141.4453
  139.4531  139.4531  139.4531
  137.4609  137.4609  137.4609
  135.4688  135.4688  135.4688
  133.4766  133.4766  133.4766
  131.4844  131.4844  131.4844
  129.4922  129.4922  129.4922
  127.5000  127.5000  127.5000
  125.5078  125.5078  125.5078
  123.5156  123.5156  123.5156
  121.5234  121.5234  121.5234
  119.5312  119.5312  119.5312
  117.5391  117.5391  117.5391
  115.5469  115.5469  115.5469
  113.5547  113.5547  113.5547
  111.5625  111.5625  111.5625
  109.5703  109.5703  109.5703
  107.5781  107.5781  107.5781
  105.5859  105.5859  105.5859
  103.5938  103.5938  103.5938
  101.6016  101.6016  101.6016
   99.6094   99.6094   99.6094
   97.6172   97.6172   97.6172
   95.6250   95.6250   95.6250
   93.6328   93.6328   93.6328
   91.6406   91.6406   91.6406
   89.6484   89.6484   89.6484
   87.6562   87.6562   87.6562
   85.6641   85.6641   85.6641
   83.6719   83.6719   83.6719
   81.6797   81.6797   81.6797
   79.6875   79.6875   79.6875
   77.6953   77.6953   77.6953
   75.7031   75.7031   75.7031
   73.7109   73.7109   73.7109
   71.7188   71.7188   71.7188
   69.7266   69.7266   69.7266
   67.7344   67.7344   67.7344
   65.7422   65.7422   65.7422
   63.7500   63.7500   63.7500
   61.7578   61.7578   61.7578
   59.7656   59.7656   59.7656
   57.7734   57.7734   57.7734
   55.7812   55.7812   55.7812
   53.7891   53.7891   53.7891
   51.7969   51.7969   51.7969
   49.8047   49.8047   49.8047
   47.8125   47.8125   47.8125
   45.8203   45.8203   45.8203
   43.8281   43.8281   43.8281
   41.8359   41.8359   41.8359
   39.8438   39.8438   39.8438
   37.8516   37.8516   37.8516
   35.8594   35.8594   35.8594
   33.8672   33.8672   33.8672
   31.8750   31.8750   31.8750
   29.8828   29.8828   29.8828
   27.8906   27.8906   27.8906
   25.8984   25.8984   25.8984
   23.9062   23.9062   23.9062
   21.9141   21.9141   21.9141
   19.9219   19.9219   19.9219
   17.9297   17.9297   17.9297
   15.9375   15.9375   15.9375
   13.9453   13.9453   13.9453
   11.9531   11.9531   11.9531
    9.9609    9.9609    9.9609
    7.9688    7.9688    7.9688
    5.9766    5.9766    5.9766
    3.9844    3.9844    3.9844
         0         0         0;         
    """)
    return

if __name__ == "__main__":
    (options,args) = myparser.parse_args()
    print options

    # FIX: validate args!

    # FIX: make the rest a function so that it can be built from inside of other python progs
    print "Opening ",options.pltsegyFile
    o = open (options.pltsegyFile,'w')

    o.write("plot\n")
    o.write("        dtfile = "+options.segyFile+";\n")

    o.write("        plfile = "+options.postscriptFile+";\n")
    o.write("        srtfile = sort."+options.pltsegyFile+".sort;\n") # FIX: pretty this up
    o.write("        cdp = t;\n")
    if options.x11:
        o.write("        device = 16; { X11.  Use 15 for postscript.\n")
    else:
        o.write("        device = 15; { Postscript.  Use 16 for X11.\n")
    o.write("        interact = 1;\n")
    o.write("        size = 550, 350;\n")
    o.write("        sort = shot;\n")
    o.write("        shb = "+str(options.shb)+"; { beginning shot\n")
    o.write("        shf = "+str(options.shf)+"; { ending shot\n")
    #o.write("        = "++";\n")
    o.write("""        noinc = 1;
        ntrab = 1;
        ntraf = 1;
        ntrinc = 1;
        speq = t;
        delt = 0.000083;
""")
    o.write("        tbi = "+str(options.tbi)+";\n")
    o.write("        tfi = "+str(options.tfi)+";\n")
    o.write("        timscal= "+str(options.timscal)+"; { suggest 1000 to 200 for chirp\n")
    o.write("        tmark = "+str(options.tmark)+";\n")
    o.write("        spa = "+str(options.spa)+";\n") #
    o.write("""        negpol= f;
        form = I6;
""")
    o.write("        anninc = "+str(options.anninc)+";\n") 
    # Is there any reason that xann should not equal shb?        
    o.write("        xann = "+str(options.shb)+";\n") 
    o.write("""        delann = 1;
        reduc = 1.00;
        clip = 1.0;
        {mxdef = -1.0;
        pscale = 30000000;
        rtl = f;
        shad = 5; {shade=5 is what worked
 """)
    writeLevel(o)
    writePalette(o)
    o.write("        chgt = 2.0;\n")
    o.write("        labhgt = 3.0;\n")
    if None == options.title:
        o.write("        label = 'l' 80000 0.255 0 '"+options.pltsegyFile+"';\n")        
    else:
        o.write("        label = 'l' 80000 0.255 0 '"+options.title+"';\n")
    o.write("        ifilc = 0;\n")
    o.write("        advplt = f;\n")
    o.write("        origin = 35 30;\n")
    o.write("end$\n")

    o.close()
    #os.chmod(options.pltsegyFile, ADD execute bit)
    # FIX: us os.chmod instead of os.system
    #os.system("chmod +x "+options.pltsegyFile)
    print "\nTo run this new script, just type this:"
    print "  pltsegy "+ options.pltsegyFile
